Attribute VB_Name = "mdlVectorMath"
Option Explicit

Public Type vec2_t
    st(2)                       As Single
End Type

Public Type vec3_t
    xyz(2)                      As Single
End Type

Public Type vec4_t
    xyzw(3)                     As Single
End Type

Public Sub matrixInvert(inm() As Single, outm() As Single)
    '-----------------------------------------------
    ' Invert a matrix. (Matrix MUST be orhtonormal!)
    '   in - Input matrix
    '   out - Output matrix
    '-----------------------------------------------
    ' Transpose rotation
    outm(0) = inm(0):    outm(1) = inm(4):    outm(2) = inm(8):
    outm(4) = inm(1):    outm(5) = inm(5):    outm(6) = inm(9):
    outm(8) = inm(2):    outm(9) = inm(6):    outm(10) = inm(10):
      
    ' Clear shearing terms
    outm(3) = 0#: outm(7) = 0#: outm(11) = 0#: outm(15) = 1#
      
    ' Translation is minus the dot of tranlation and rotations
    outm(12) = -(inm(12) * inm(0)) - (inm(13) * inm(1)) - (inm(14) * inm(2)):
    outm(13) = -(inm(12) * inm(4)) - (inm(13) * inm(5)) - (inm(14) * inm(6)):
    outm(14) = -(inm(12) * inm(8)) - (inm(13) * inm(9)) - (inm(14) * inm(10)):
End Sub

Public Sub vecCopy(vecIn() As Single, vecOut() As Single)
    vecOut(0) = vecIn(0)
    vecOut(1) = vecIn(1)
    vecOut(2) = vecIn(2)
End Sub

Public Sub vecMatMult(vecIn() As Single, m() As Single, vecOut() As Single)
    '-------------------------------
    ' Multiply a vector by a matrix.
    '   vecIn - Input vector
    '   m - Input matrix
    '   vecOut - Output vector
    '-------------------------------
    vecOut(0) = (vecIn(0) * m(0)) + (vecIn(1) * m(4)) + (vecIn(2) * m(8)) + m(12):
    vecOut(1) = (vecIn(0) * m(1)) + (vecIn(1) * m(5)) + (vecIn(2) * m(9)) + m(13):
    vecOut(2) = (vecIn(0) * m(2)) + (vecIn(1) * m(6)) + (vecIn(2) * m(10)) + m(14):
End Sub

Public Function vecNormalize(vec() As Single) As Single
    Dim mag As Single
    mag = Sqr(vec(0) * vec(0) + _
                      vec(1) * vec(1) + _
                      vec(2) * vec(2))

    ' don't divide by zero
    If (mag > -0.0000001) And (mag < 0.0000001) _
       Then
           vec(0) = 0#
           vec(1) = 0#
           vec(2) = 0#
           Exit Function
    End If

    vec(0) = vec(0) / mag
    vec(1) = vec(1) / mag
    vec(2) = vec(2) / mag
   
    vecNormalize = mag

End Function


Public Sub vecMat3x3Mult(vecIn() As Single, m() As Single, vecOut() As Single)
    '-------------------------------------------------------
    ' Multiply a vector by just the 3x3 portion of a matrix.
    '   vecIn - Input vector
    '   m - Input matrix
    '   vecOut - Output vector
    '-------------------------------------------------------
    vecOut(0) = (vecIn(0) * m(0)) + (vecIn(1) * m(4)) + (vecIn(2) * m(8)):
    vecOut(1) = (vecIn(0) * m(1)) + (vecIn(1) * m(5)) + (vecIn(2) * m(9)):
    vecOut(2) = (vecIn(0) * m(2)) + (vecIn(1) * m(6)) + (vecIn(2) * m(10)):
End Sub

Public Sub matrixIdentity(matrix() As Single)
    '-------------------------------
    ' Set matrix to identity
    '   matrix - Input/Output matrix
    '-------------------------------
    matrix(0) = 1#:    matrix(1) = 0#:    matrix(2) = 0#:    matrix(3) = 0#:
    matrix(4) = 0#:    matrix(5) = 1#:    matrix(6) = 0#:    matrix(7) = 0#:
    matrix(8) = 0#:    matrix(9) = 0#:    matrix(10) = 1#:   matrix(11) = 0#:
    matrix(12) = 0#:   matrix(13) = 0#:   matrix(14) = 0#:   matrix(15) = 1#:
End Sub
